#include <vkcv/Core.hpp>
#include <vkcv/Pass.hpp>
#include <vkcv/shader/GLSLCompiler.hpp>
#include <vkcv/camera/CameraManager.hpp>

#include <glm/glm.hpp>

int main(int argc, const char** argv) {
  const std::string applicationName = "My application";
  
  vkcv::Core core = vkcv::Core::create(
    applicationName,
    VK_MAKE_VERSION(0, 0, 1),
    { vk::QueueFlagBits::eGraphics },
    { VK_KHR_SWAPCHAIN_EXTENSION_NAME }
  );
  
  const int windowWidth = 800;
  const int windowHeight = 600;
  const bool isWindowResizable = false;
  
  vkcv::WindowHandle windowHandle = core.createWindow(
    applicationName,
    windowWidth,
    windowHeight,
    isWindowResizable
  );
  
  vkcv::Window& window = core.getWindow(windowHandle);
  
  vkcv::PassHandle renderPass = vkcv::passSwapchain(
    core,
    window.getSwapchain(),
    { vk::Format::eUndefined }
  );
  
  vkcv::ShaderProgram shaderProgram;
  vkcv::shader::GLSLCompiler compiler;
  
  compiler.compileProgram(
    shaderProgram,
    {
      { vkcv::ShaderStage::VERTEX, "shaders/shader.vert" },
      { vkcv::ShaderStage::FRAGMENT, "shaders/shader.frag" }
    },
    nullptr
  );
  
  vkcv::GraphicsPipelineHandle gfxPipeline = core.createGraphicsPipeline(
    vkcv::GraphicsPipelineConfig(
      shaderProgram,
      renderPass,
      {},
      {}
    )
  );
  
  vkcv::VertexData vertexData;
  vertexData.setCount(3);
  
  vkcv::InstanceDrawcall drawcall (vertexData);
  const auto swapchainInput = vkcv::ImageHandle::createSwapchainImageHandle();
  
  vkcv::camera::CameraManager cameraManager (window);
  
  auto camHandle = cameraManager.addCamera(vkcv::camera::ControllerType::PILOT);
  cameraManager.getCamera(camHandle).setPosition(glm::vec3(0, 0, 2));
  
  core.run([&](const vkcv::WindowHandle &windowHandle,
               double t,
               double dt,
               uint32_t swapchainWidth,
               uint32_t swapchainHeight
  ) {
    cameraManager.update(dt);
    
    glm::mat4 mvp = cameraManager.getActiveCamera().getMVP();
    
    auto cmdStream = core.createCommandStream(vkcv::QueueType::Graphics);
    
    core.recordDrawcallsToCmdStream(
      cmdStream,
      gfxPipeline,
      vkcv::pushConstants<glm::mat4>(mvp),
      { drawcall },
      { swapchainInput },
      windowHandle
    );
    
    core.prepareSwapchainImageForPresent(cmdStream);
    core.submitCommandStream(cmdStream);
  });
}
